@@ -17,14 +17,14 @@ package Data::FormValidator::Results;
 
 use Carp;
 use Symbol;
-use Data::FormValidator::Filters qw/:filters/;
-use Data::FormValidator::Constraints (qw/:validators :matchers/);
+use Data::FormValidator::Filters ':filters';
+use Data::FormValidator::Constraints qw(:validators :matchers);
 use vars qw/$AUTOLOAD $VERSION/;
 use overload
   'bool' => \&_bool_overload_based_on_success,
   fallback => 1;
 
-$VERSION = 4.14;
+$VERSION = 4.50;
 
 =pod
 
@@ -121,7 +121,7 @@ sub _process {
 		}
 	}
 
-	# Apply inconditional filters
+	# Apply unconditional filters
     foreach my $filter (_arrayify($profile->{filters})) {
 		if (defined $filter) {
 			# Qualify symbolic references
@@ -169,6 +169,9 @@ sub _process {
 		}
 	}
  
+    # store the filtered data away for later use
+    $self->{__FILTERED_DATA} = \%valid;
+ 
     my %required    = map { $_ => 1 } _arrayify($profile->{required});
     my %optional    = map { $_ => 1 } _arrayify($profile->{optional});
 
@@ -218,7 +221,7 @@ sub _process {
 			if (ref($deps) eq 'HASH') {
 				foreach my $key (keys %$deps) {
                     # Handle case of a key with a single value given as an arrayref
-                    # There is probably a better, more general soution to this problem.
+                    # There is probably a better, more general solution to this problem.
                     my $val_to_compare;
                     if ((ref $valid{$field} eq 'ARRAY') and (scalar @{ $valid{$field} } == 1)) {
                         $val_to_compare = $valid{$field}->[0];
@@ -262,8 +265,22 @@ sub _process {
 		delete $valid{$field};
 	}
 
+    # Add defaults from defaults_regexp_map
+    my %private_defaults;
+    my @all_possible = keys %optional, keys %required, keys %require_some;
+    while ( my ($re,$value) = each %{$profile->{defaults_regexp_map}} ) {
+        # We only add defaults for known fields. 
+        for (@all_possible) {
+            $private_defaults{$_} = $value if m/$re/;
+        }
+    }
+
     # Fill defaults
-    while ( my ($field,$value) = each %{$profile->{defaults}} ) {
+    my %combined_defaults = ( 
+        %private_defaults, 
+        %{ $profile->{defaults} || {} } 
+    );
+    while ( my ($field,$value) = each %combined_defaults ) {
         unless(exists $valid{$field}) {
             if (ref($value) && ref($value) eq "CODE") {
                 $valid{$field} = $value->($self);
@@ -406,7 +423,7 @@ array ref if the field had multiple values:
 
  $value = $r->valid('field');
 
-If called with one argument in array conect, it returns the values of C<field> 
+If called with one argument in array context, it returns the values of C<field> 
 as an array:
 
  @values = $r->valid('field');
@@ -710,6 +727,11 @@ sub get_input_data {
     }
 }
 
+sub get_filtered_data {
+    my $self = shift;
+	return $self->{__FILTERED_DATA};
+}
+
 sub get_current_constraint_field {
 	my $self = shift;
 	return $self->{__CURRENT_CONSTRAINT_FIELD};
@@ -725,6 +747,14 @@ sub get_current_constraint_name {
 	return $self->{__CURRENT_CONSTRAINT_NAME};
 }
 
+sub untainted_constraint_value {
+    my $self = shift;
+    my $match = shift;
+
+    return undef unless defined $match;
+    return $self->{__UNTAINT_THIS} ? $match : length $match;
+}
+
 sub set_current_constraint_name {
 	my $self = shift;
 	my $value = shift;
@@ -739,7 +769,7 @@ sub name_this {
 
 # INPUT: prefix_string, hash reference
 # Copies the hash and prefixes all keys with prefix_string
-# OUTPUT: hash refence
+# OUTPUT: hash reference
 sub prefix_hash {
 	my ($pre,$href) = @_;
 	die "prefix_hash: need two arguments" unless (scalar @_ == 2);
@@ -982,6 +1012,10 @@ sub _constraint_check_match {
 	my 	($self,$c,$params,$untaint_this) = @_;
 	die "_constraint_check_match received wrong number of arguments" unless (scalar @_ == 4);
 
+    # Store whether or not we want untainting in the object so that constraints
+    # can do the right thing conditionally.
+    $self->{__UNTAINT_THIS} = $untaint_this;
+
     my $match = $c->{constraint}->( @$params );
 
     # We need to make this distinction when untainting,
@@ -1042,7 +1076,7 @@ sub _get_input_as_hash {
 #	params => [ \'zoo' ]
 #  }
 #
-# Still, it's possble, the two bits of logic could be refactored into one location if you cared
+# Still, it's possible, the two bits of logic could be refactored into one location if you cared
 # to do that. 
 
 sub _create_regexp_common_constraint  {
@@ -1087,7 +1121,7 @@ sub _add_constraints_from_map {
 					my $new = $profile->{$map_name}{$re};
 					# If they already have an arrayref of constraints, add to the list
 					if (ref $cur eq 'ARRAY') {
-						push @{ $result{$key} }, $new;
+                        push @{ $result{$key} }, @$cur, $new;
 					} 
 					# If they have a single constraint defined, create an array ref with with this plus the new one
 					elsif ($cur) {
